<?php
namespace tests\Zeedhi\Tools\DatabaseReader\Adapters;

use Zeedhi\Tools\DatabaseReader\DatabaseReader;
use Zeedhi\Tools\DatabaseReader\DTO\Column;
use Zeedhi\Tools\DatabaseReader\DTO\Constraints\ForeignKey;
use Doctrine\DBAL\DriverManager;

class MySQLTest extends \PHPUnit_Framework_TestCase {

    /** @var DatabaseReader */
    protected $reader;

    protected function setUp() {
        if (!extension_loaded('pdo_mysql')) {
            $this->markTestSkipped('The ' . __CLASS__ . ' requires the use of pdo_mysql extension.');
        }

        $connectionToRead = DriverManager::getConnection(array(
            'driver'    => 'pdo_mysql',
            'user'      => 'root',
            'password'  => 'teknisa',
            'host'      => '192.168.122.57',
            'port'      => '3306',
            'dbname'    => 'sakila'
        ));
        $this->reader = DatabaseReader::factoryFromConnection($connectionToRead);
    }


    public function testGetAllTableNames() {
        $expectedTableNames = array(
            'actor',
			'actor_info',
			'address',
			'category',
			'city',
			'country',
			'customer',
			'customer_list',
			'film',
			'film_actor',
			'film_category',
			'film_list',
			'film_text',
			'inventory',
			'language',
			'nicer_but_slower_film_list',
			'payment',
			'rental',
			'sales_by_film_category',
			'sales_by_store',
			'staff',
			'staff_list',
			'store',
			'useless'
        );

        $tableNames = $this->reader->getAllTableNames();
        foreach ($expectedTableNames as $expectedTableName) {
            $this->assertContains($expectedTableName, $tableNames, "Table expected to be found.");
        }

        $this->assertCount(count($expectedTableNames), $tableNames, "Number of expected tables found.");
    }

    public function testGetTableThatDoesNotExist() {
        $tableName = "INVALID_NAME";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} was not found in current database.");
        $this->reader->getTable($tableName);
    }

    public function testGetTable() {
        $DTOTable = $this->reader->getTable("city");
        $this->assertInstanceOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Table", $DTOTable, "Return must be a instance of DTO\\TAble.");
        $this->assertEquals("city", $DTOTable->getName(), "Table name must be 'city'.");
    }

    public function assertNullOrNumeric($value, $message = '') {
        $this->assertTrue(is_numeric($value) || is_null($value), $message);
    }

    public function assertDTOColumn(Column $DTOColumn) {
        $columnTypeList = array(
            Column::DATA_TYPE_BINARY,
            Column::DATA_TYPE_BOOLEAN,
            Column::DATA_TYPE_DATE,
            Column::DATA_TYPE_DATETIME,
            Column::DATA_TYPE_FLOAT,
            Column::DATA_TYPE_INTEGER,
            Column::DATA_TYPE_STRING,
        );
        $this->assertNotEmpty($DTOColumn->getName(), "Column name can't be empty.");
        $this->assertNullOrNumeric($DTOColumn->getDataPrecision(), "DataPrecision must be null or numeric value.");
        $this->assertNullOrNumeric($DTOColumn->getDataLength(), "DataLength must be null or numeric value.");
        $this->assertNullOrNumeric($DTOColumn->getDataScale(), "DataScale must be null or numeric value.");
        $this->assertContains($DTOColumn->getNullAble(), array("Y", "N"), "Column nullable must be a boolean");
        $this->assertContains($DTOColumn->getDataType(), $columnTypeList, "Column must have a DataType in the list.");
    }

    public function testGetColumnsFromTable() {
        $DTOColumns = $dbReader = $this->reader->getColumnsFromTable("city");
        $this->assertContainsOnlyInstancesOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Column", $DTOColumns);
        foreach($DTOColumns as $DTOColumn) {
            $this->assertDTOColumn($DTOColumn);
        }
    }

    public function testGetColumnsFromInvalidTable() {
        $tableName = "INVALID_NAME";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} was not found in current database.");
        $this->reader->getColumnsFromTable($tableName);
    }

    public function testGetPrimaryKeyFromTable() {
        $primaryKey = $this->reader->getPrimaryKeyFromTable("city");
        $this->assertInstanceOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Constraints\\PrimaryKey", $primaryKey);
        $this->assertConsColumnCollection($primaryKey->getConsColumns());
    }

    public function testGetPrimaryKeyFromInvalidTable() {
        $tableName = "INVALID_NAME";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} was not found in current database.");
        $this->reader->getPrimaryKeyFromTable($tableName);
    }

    public function testTableWithoutPrimaryKey() {
        $tableName = "useless";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} have not got a primary key.");
        $this->reader->getPrimaryKeyFromTable($tableName);

    }

    /**
     * @param ForeignKey $foreignKey
     */
    public function assertForeignKey($foreignKey) {
        static $deleteRuleList = array(
            ForeignKey::DELETE_RULE_CASCADE,
            ForeignKey::DELETE_RULE_RESTRICT,
            ForeignKey::DELETE_RULE_SET_NULL,
            ForeignKey::DELETE_RULE_NO_ACTION,
        );

        $this->assertConsColumnCollection($foreignKey->getConsColumns());
        $this->assertNotNull($foreignKey->getReferencedConstraintName());
        $this->assertContains($foreignKey->getDeleteRule(), $deleteRuleList, "ForeignKey must have a delete rule in list.");
    }

    public function testGetForeignKeyFromTable() {
        $foreignKeys = $this->reader->getForeignKeysFromTable("city");
        $this->assertContainsOnlyInstancesOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Constraints\\ForeignKey", $foreignKeys);
        foreach ($foreignKeys as $foreignKey) {
            $this->assertForeignKey($foreignKey);
        }
    }

    public function testGetForeignKeyFromInvalidTable() {
        $tableName = "INVALID_NAME";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} was not found in current database.");
        $this->reader->getForeignKeysFromTable($tableName);
    }

    public function testGetUniqueConstraintFromTable() {
        $uniqueConstraints = $this->reader->getUniqueConstraintsFromTable("city");
        $this->assertContainsOnlyInstancesOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Constraints\\Unique", $uniqueConstraints);
        foreach ($uniqueConstraints as $uniqueConstraint) {
            $this->assertConsColumnCollection($uniqueConstraint->getConsColumns());
        }
    }

    public function testGetUniqueConstraintFromInvalidTable() {
        $tableName = "INVALID_NAME";
        $this->setExpectedException("\\Zeedhi\\Tools\\DatabaseReader\\Exception", "Table {$tableName} was not found in current database.");
        $this->reader->getUniqueConstraintsFromTable($tableName);
    }

    /**
     * @param \Zeedhi\Tools\DatabaseReader\DTO\Constraints\ConsColumn[] $DTOConsColumns
     */
    public function assertConsColumnCollection($DTOConsColumns) {
        $this->assertContainsOnlyInstancesOf("\\Zeedhi\\Tools\\DatabaseReader\\DTO\\Constraints\\ConsColumn", $DTOConsColumns);
        foreach ($DTOConsColumns as $DTOConsColumn) {
            $this->assertNotEmpty($DTOConsColumn->getColumnName());
            $this->assertTrue(is_numeric($DTOConsColumn->getPosition()));
        }
    }
}